function [gdat_data,gdat_params,error_status,varargout] = gdat_aug(shot,data_request,varargin)
%
% function [gdat_data,gdat_params,error_status,varargout] = gdat(shot,data_request,varargin)
%
% Aim: get data from a given machine using full path or keywords. 
%      data_request are and should be case independent (transformed in lower case in the function and outputs)
%
% If no inputs are provided, return the list of available pre-defined data_request in gdat_data and default parameters gdat_params
%
% Inputs:
%
%    no inputs: return default parameters in a structure form in gdat_params
%    shot: shot number
%    data_request: keyword (like 'ip') or trace name or structure containing all parameters but shot number
%    varargin{i},varargin{i+1},i=1:nargin-2: additional optional parameters given in pairs: param_name, param_value
%                                            The optional parameters list might depend on the data_request
%              examples of optional parameters:
%                               'plot',1 (plot is set by default to 0)
%                               'machine','AUG' (the default machine is the local machine)
%
%
% Outputs:
%
% gdat_data: structure containing the data, the time trace if known and other useful information
% gdat_data.t : time trace
% gdat_data.data: requested data values
% gdat_data.dim : values of the various coordinates related to the dimensions of .data(:,:,...)
%                     note that one of the dim is the time, replicated in .t for clarity
% gdat_data.dimunits : units of the various dimensions, 'dimensionless' if dimensionless
% gdat_data.error_bar : if provided
% gdat_data.gdat_call : list of parameters provided in the gdat call (so can be reproduced)
% gdat_data.shot: shot number
% gdat_data.machine: machine providing the data
% gdat_data.gdat_request: keyword for gdat if relevant
% gdat_data.data_fullpath: full path to the data node if known and relevant, or relevant expression called if relevant
% gdat_data.gdat_params: copy gdat_params for completeness
% gdat_data.xxx: any other relevant information
%
%
% Examples:
% (should add working examples for various machines (provides working shot numbers for each machine...))
% 
%    [a1,a2]=gdat;
%    a2.data_request = 'Ip';
%    a3=gdat(48836,a2);  % gives input parameters as a structure, allows to call the same for many shots
%    a4=gdat('opt1',123,'opt2',[1 2 3],'shot',48832,'data_request','Ip','opt3','aAdB'); % all in pairs
%    a5=gdat(48836,'ip'); % standard call
%    a6=gdat(48836,'ip','Opt1',123,'Doplot',1,'opt2','Abc'); % standard call with a few options (note all lowercase in output)

%
% Comments for local developer:
% This gdat is just a "header routine" calling the gdat for the specific machine gdat_`machine`.m which can be called
% directly, thus which should be able to treat the same type of input arguments
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prepare some variables, etc

varargout{1}=cell(1,1);
error_status=1;
nverbose = 1;

% construct default parameters structure
gdat_params.data_request = '';
default_machine = 'aug';

gdat_params.machine=default_machine;
gdat_params.doplot = 0;
gdat_params.exp_name = 'AUGD';
gdat_params.equil = 'EQI';

% construct list of keywords from global set of keywords and specific AUG set
% get data_request names from centralized table
data_request_names = get_data_request_names;

% add AUG specific to all:
if ~isempty(data_request_names.aug)
  aug_names = fieldnames(data_request_names.aug);
  for i=1:length(aug_names)
    data_request_names.all.(aug_names{i}) = data_request_names.aug.(aug_names{i});
  end
end
data_request_names_all = fieldnames(data_request_names.all);

% $$$ data_request_names_all= [{'ip'} {'b0'} {'zmag'} {'rmag'} {'rcont'} {'zcont'} {'vol'} {'rhovol'} {'qrho'} {'q95'} {'kappa'} ...
% $$$       {'delta'} {'deltatop'} {'deltabot'} {'neint'} {'nel'} ...
% $$$       {'ne'} {'te'} {'nerho'} {'terho'}  {'ne_edge'} {'te_edge'} {'nerho_edge'} {'terho_edge'} {'nerhozshift'} {'terhozshift'} {'profnerho'} {'profterho'} ...
% $$$       {'neft'} {'teft'} {'neftav'} {'teftav'} {'neft:trial'} {'teft:trial'} {'neftav:trial'} {'teftav:trial'}  ...
% $$$       {'sxr'} {'sxr'} {'ece'} {'mpx'} {'ioh'} {'vloop'} {'pgyro'} {'jtor'} {'vi_tor'} {'vi_torfit'} {'vi_pol'} {'vi_polfit'} {'ti'} {'tifit'} {'ni'} {'nifit'} {'zeffcxrs'} {'zeffcxrsfit'}];

% construct default output structure
gdat_data.data = [];
gdat_data.units = [];
gdat_data.dim = [];
gdat_data.dimunits = [];
gdat_data.t = [];
gdat_data.x = [];
gdat_data.shot = [];
gdat_data.gdat_request = [];
gdat_data.gdat_params = gdat_params;
gdat_data.data_fullpath = [];


% Treat inputs:
ivarargin_first_char = 3;
data_request_eff = '';
if nargin>=2 && ischar(data_request); data_request = lower(data_request); end

gdat_data.gdat_request = data_request_names_all; % so if return early gives list of possible request names
% no inputs
if nargin==0
  % return defaults and list of keywords
  return
end

do_mdsopen_mdsclose = 1;
% treat 1st arg
if nargin>=1
  if isempty(shot)
    % means mdsopen(shot) already performed
% $$$     shot = mdsipmex(2,'$SHOT');
% $$$     gdat_data.shot = shot;
    shot=0;
    do_mdsopen_mdsclose = 0;
  elseif isnumeric(shot)
    gdat_data.shot = shot;
  elseif ischar(shot)
    ivarargin_first_char = 1;
  else
    warning('type of 1st argument unexpected, should be numeric or char')
    error_status=2;
    return
  end
  if nargin==1
    % Only shot number given. If there is a default data_request set it and continue, otherwise return
    return
  end
end
% 2nd input argument if not part of pairs
if nargin>=2 && ivarargin_first_char~=1
  if isempty(data_request)
    return
  end
  % 2nd arg can be a structure with all options except shot_number, or a string for the pathname or keyword, or the start of pairs string/value for the parameters
  if isstruct(data_request)
    if ~isfield(data_request,'data_request')
      warning('expects field data_request in input parameters structure')
      error_status=3;
      return
    end
    %data_request.data_request = lower(data_request.data_request);
    data_request_eff = data_request.data_request;
    gdat_params = data_request;
  else
    % since data_request is char check from nb of args if it is data_request or a start of pairs
    if mod(nargin-1,2)==0
      ivarargin_first_char = 2;
    else
      ivarargin_first_char = 3;
      data_request_eff = data_request;
    end
  end
end

if ~isstruct(data_request)
  gdat_params.data_request = data_request_eff;
end

% if start pairs from shot or data_request, shift varargin
if ivarargin_first_char==1
  varargin_eff{1} = shot;
  varargin_eff{2} = data_request;
  varargin_eff(3:nargin) = varargin(:);
elseif ivarargin_first_char==2
  varargin_eff{1} = data_request;
  varargin_eff(2:nargin-1) = varargin(:);
else
  varargin_eff(1:nargin-2) = varargin(:);
end

% extract parameters from pairs of varargin:
if (nargin>=ivarargin_first_char)
  if mod(nargin-ivarargin_first_char+1,2)==0
    for i=1:2:nargin-ivarargin_first_char+1
      if ischar(varargin_eff{i})
        % enforce lower case for any character driven input
        if ischar(varargin_eff{i+1})
          gdat_params.(lower(varargin_eff{i})) = lower(varargin_eff{i+1});
        else
          gdat_params.(lower(varargin_eff{i})) = varargin_eff{i+1};
        end
      else
        warning(['input argument nb: ' num2str(i) ' is incorrect, expects a character string'])
        error_status=401;
        return
      end
    end
  else
    warning('number of input arguments incorrect, cannot make pairs of parameters')
    error_status=402;
    return
  end
end
data_request_eff = gdat_params.data_request; % in case was defined in pairs

% if it is a request_keyword can obtain description:
if length(data_request_eff)==1
  ij=strmatch(data_request_eff,data_request_names_all,'exact');
else
  ij=[];
end
gdat_data.gdat_request = data_request_eff;

if ~isempty(ij); 
  gdat_data.gdat_request = data_request_names_all{ij};
  if isfield(data_request_names.all.(data_request_names_all{ij}),'description') && ~isempty(data_request_names.all.(data_request_names_all{ij}).description)
    % copy description of keyword
    gdat_data.request_description = data_request_names.all.(data_request_names_all{ij}).description;
  end
end

% special treatment if shot and data_request given within pairs
if isfield(gdat_params,'shot')
  shot = gdat_params.shot; % should use only gdat_params.shot but change shot to make sure
  gdat_data.shot = gdat_params.shot;
  gdat_params=rmfield(gdat_params,'shot');
end
if ~isfield(gdat_params,'data_request') || isempty(gdat_params.data_request)
  % warning('input for ''data_request'' missing from input arguments') % might be correct, asking for list of requests
  error_status=5;
  return
end
gdat_data.gdat_params = gdat_params;

% re-assign main variables to make sure use the one in gdat_data structure
shot = gdat_data.shot;
data_request_eff = gdat_data.gdat_params.data_request;
error_status = 6; % at least reached this level

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Specifications on how to get the data provided in aug_requests_mapping
mapping_for_aug = aug_requests_mapping(data_request_eff);
gdat_data.label = mapping_for_aug.label;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1st treat the simplest method: "signal"
if strcmp(mapping_for_aug.method,'signal')
  exp_location = gdat_data.gdat_params.exp_name;
  if ~isempty(mapping_for_aug.expression) && iscell(mapping_for_aug.expression) && length(mapping_for_aug.expression)>=3
    exp_location = mapping_for_aug.expression{3};
  elseif ~isempty(mapping_for_aug.expression) && length(mapping_for_aug.expression)>=2
    mapping_for_aug.expression{3} = exp_location;
  else
    error_status = 101;
    disp(['expects at least 2 cells in expression, mapping_for_aug.expression = ' mapping_for_aug.expression]);
    return
  end
  gdat_data.gdat_params.exp_name = exp_location;
  [aatmp,error_status]=rdaAUG_eff(shot,mapping_for_aug.expression{1},mapping_for_aug.expression{2},exp_location);
  if error_status~=0
    if nverbose>=3; disp(['error after rdaAUG in signal with data_request_eff= ' data_request_eff]); end
    return
  end
  gdat_data.data = aatmp.data;
  gdat_data.t = aatmp.t;
  if isempty(aatmp.data)
    return
  end
  if mapping_for_aug.timedim<=0
    % need to guess timedim
    if prod(size(aatmp.data)) == length(aatmp.data)
      mapping_for_aug.timedim = 1;
    elseif length(size(aatmp.data))==2
      % 2 true dimensions
      if length(aatmp.t) == size(aatmp.data,1)
	mapping_for_aug.timedim = 1;
      else
	mapping_for_aug.timedim = 2;
      end
    else
      % more than 2 dimensions, find the one with same length to define timedim
      mapping_for_aug.timedim = find(size(aatmp.data)==length(aatmp.t));
      if length(mapping_for_aug.timedim); mapping_for_aug.timedim = mapping_for_aug.timedim(1); end
    end
    mapping_for_aug.gdat_timedim = mapping_for_aug.timedim
  end
  if length(size(aatmp.data))==1 | (length(size(aatmp.data))==2 & size(aatmp.data,2)==1)
    gdat_data.dim=[{aatmp.t}];
    gdat_data.dimunits={'time [s]'};
  elseif length(size(aatmp.data))==2
    gdat_data.dim{mapping_for_aug.timedim} = gdat_data.t;
    gdat_data.dimunits{mapping_for_aug.timedim} = 'time [s]';
    gdat_data.dim{setdiff([1:2],mapping_for_aug.timedim)} = gdat_data.x;
  else
    gdat_data.dim{mapping_for_aug.timedim} = gdat_data.t;
    gdat_data.dimunits{mapping_for_aug.timedim} = 'time [s]';
    nbdims = length(size(gdat_data.data));
    for i=1:nbdims
      if i~=mapping_for_aug.timedim
	ieff=i;
	if i>mapping_for_aug.timedim; ieff=i-1; end
	gdat_data.dim{i} = gdat_data.x{ieff};
      end
    end
  end
  nbdims = length(gdat_data.dim);
  gdat_data.units = aatmp.units;
  if mapping_for_aug.gdat_timedim>0 && mapping_for_aug.gdat_timedim ~= mapping_for_aug.timedim
    % shift timedim to gdat_timedim data(i,j,...itime,k,...) -> data(i,inewtime,j,...,k,...)
    % note that this means that gdat_data.x and gdat_data.t are same and correct, 
    % only .data, .dim and .dimunits need to be changed
    iprev=[1:nbdims];
    ij=find(dim_nontim>mapping_for_aug.gdat_timedim-1);
    inew=[1:mapping_for_aug.gdat_timedim-1 mapping_for_aug.timedim dim_nontim(ij)];
    data_sizes = size(aatmp.data);
    gdat_data.data = NaN*ones(data_sizes(inew));
    abcol=ones(1,nbdims)*double(':'); abcomma=ones(1,nbdims)*double(',');
    dimstr_prev=['(' repmat(':,',1,mapping_for_aug.timedim-1) 'it,' ...
                 repmat(':,',1,nbdims-mapping_for_aug.timedim-1) ':)'];
    dimstr_new=['(' repmat(':,',1,mapping_for_aug.gdat_timedim-1) 'it,' ...
                repmat(':,',1,nbdims-mapping_for_aug.gdat_timedim-1) ':)'];
    % eval gdat_data.data(;,:,...,it,...) = aatmp.data(:,:,:,it,...);
    for it=1:size(aatmp.data,mapping_for_aug.timedim)
      shift_eval = ['gdat_data.data' dimstr_new ' = aatmp.data'  dimstr_prev ';'];
      eval(shift_eval);
    end
    gdat_data.dim = aatmp.dim(inew);
    % gdat_data.dimunits = aatmp.dimunits(inew);
  end
  gdat_data.data_fullpath=mapping_for_aug.expression;
  % end of method "signal"

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif strcmp(mapping_for_aug.method,'expression')
  % 2nd: method="expression"
  % assume expression contains an expression to evaluate and which cretaes a local structure into variable gdat_tmp
  % we copy the structure, to make sure default nodes are defined and to avoid if return is an closed object like tdi
  % eval_expr = [mapping_for_aug.expression ';'];
  eval([mapping_for_aug.expression ';']);
  if isempty(gdat_tmp) || (~isstruct(gdat_tmp) & ~isobject(gdat_tmp))
    warning(['expression does not create a gdat_tmp structure: ' mapping_for_aug.expression])
    error_status=801;
    return
  end
  tmp_fieldnames = fieldnames(gdat_tmp);
  if sum(strcmp(tmp_fieldnames,'data'))==0 % note: cannot do isfield since gdat_tmp might be an object
    warning(['expression does not return a child name ''data'' for ' data_request_eff])
  end
  for i=1:length(tmp_fieldnames)
    gdat_data.(tmp_fieldnames{i}) = gdat_tmp.(tmp_fieldnames{i});
  end
  % add .t and .x in case only dim is provided
  % do not allow shifting of timedim since should be treated in the relevant expression
  ijdim=find(strcmp(tmp_fieldnames,'dim')==1);
  if ~isempty(ijdim)
    nbdims = length(gdat_data.dim);
    if mapping_for_aug.timedim==-1; 
      mapping_for_aug.timedim = nbdims;
      if (size(gdat_data.data,nbdims)==1 && nbdims>1); mapping_for_aug.timedim = nbdims-1; end
    end
    dim_nontim = setdiff([1:nbdims],mapping_for_aug.timedim);
    ijt=find(strcmp(tmp_fieldnames,'t')==1);
    if isempty(ijt)
      gdat_data.t = gdat_data.dim{mapping_for_aug.timedim};
    end
    ijx=find(strcmp(tmp_fieldnames,'x')==1);
    if isempty(ijx)
      if ~isempty(dim_nontim)
        % since most cases have at most 2d, copy as array if data is 2D and as cell if 3D or more
        if length(dim_nontim)==1
          gdat_data.x = gdat_data.dim{dim_nontim(1)};
        else
          gdat_data.x = gdat_data.dim(dim_nontim);
        end
      end
    end
    gdat_data.data_fullpath=mapping_for_aug.expression;
  end
  % end of method "expression"

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif strcmp(mapping_for_aug.method,'switchcase')
  switch data_request_eff % not lower(...) since data_request_eff should be lower case already at this stage
   case {'eqdsk'}
    %
    time_eqdsks=1.; % default time
    if isfield(gdat_data.gdat_params,'time') && ~isempty(gdat_data.gdat_params.time)
      time_eqdsks = gdat_data.gdat_params.time;
    else
      gdat_data.gdat_params.time = time_eqdsks;
      disp(['"time" is expected as an option, choose default time = ' num2str(time_eqdsks)]);
    end
    gdat_data.gdat_params.time = time_eqdsks;
    gdat_data.t = time_eqdsks;
    zshift = 0.;
    if isfield(gdat_data.gdat_params,'zshift') && ~isempty(gdat_data.gdat_params.zshift)
      zshift = gdat_data.gdat_params.zshift;
    else
      gdat_data.gdat_params.zshift = zshift;
    end
    gdat_data.gdat_params.zshift = zshift;
    params_equil = gdat_data.gdat_params;
    params_equil.data_request = 'equil';
    [equil,params_equil,error_status] = gdat_aug(shot,params_equil);
    if error_status>0
      if nverbose>=3; disp(['problems with ' params_equil.data_request]); end
      return
    end
    gdat_data.gdat_params = params_equil;
    gdat_data.equil = equil;
    for itime=1:length(time_eqdsks)
      time_eff = time_eqdsks(itime);
      % use read_results updated to effectively obtain an eqdsk with sign correct with COCOS=2
      [eqdskAUG, equil_all_t, equil_t_index]=geteqdskAUG(equil,time_eff,gdat_data.gdat_params.zshift);
      eqdskAUG.fnamefull = fullfile(['/tmp/' getenv('USER')],['EQDSK_' num2str(shot) 't' num2str(time_eff,'%.4f')]);
      cocos_out = equil.cocos;
      if isfield(gdat_data.gdat_params,'cocos') && ~isempty(gdat_data.gdat_params.cocos)
        cocos_out = gdat_data.gdat_params.cocos;
      else
        gdat_data.gdat_params.cocos = cocos_out;
      end
      if equil.cocos ~= cocos_out
	[eqdsk_cocosout, eqdsk_cocosout_IpB0pos,cocos_inout]=eqdsk_cocos_transform(eqdskAUG,[equil.cocos cocos_out]);
      else
	eqdsk_cocosout = eqdskAUG;
      end
      % for several times, use array of structure for eqdsks, 
      % cannot use it for psi(R,Z) in .data and .x since R, Z might be different at different times,
      % so project psi(R,Z) on Rmesh, Zmesh of 1st time
      if length(time_eqdsks) > 1
        gdat_data.eqdsk{itime} = write_eqdsk(eqdskAUG.fnamefull,eqdsk_cocosout);
        if itime==1
          gdat_data.data(:,:,itime) = gdat_data.eqdsk{itime}.psi;
          gdat_data.dim{1} = gdat_data.eqdsk{itime}.rmesh;
          gdat_data.dim{2} = gdat_data.eqdsk{itime}.zmesh;
        else
	  xx=repmat(reshape(gdat_data.dim{1},length(gdat_data.dim{1}),1),1,size(gdat_data.eqdsk{itime}.psi,2));
	  yy=repmat(reshape(gdat_data.dim{2},1,length(gdat_data.dim{2})),size(gdat_data.eqdsk{itime}.psi,1),1);
	  aa = interpos2Dcartesian(gdat_data.eqdsk{itime}.rmesh,gdat_data.eqdsk{itime}.zmesh ...
	  ,gdat_data.eqdsk{itime}.psi,xx,yy,-1,-1);
          gdat_data.data(:,:,itime) = aa;
        end
      else
        gdat_data.eqdsk = write_eqdsk(eqdskAUG.fnamefull,eqdsk_cocosout);
        gdat_data.data = gdat_data.eqdsk.psi;
        gdat_data.dim{1} = gdat_data.eqdsk.rmesh;
        gdat_data.dim{2} = gdat_data.eqdsk.zmesh;
      end
    end
    gdat_data.dim{3} = gdat_data.t;
    gdat_data.x = gdat_data.dim(1:2);
    gdat_data.data_fullpath=['psi(R,Z) and eqdsk from geteqdskAUG with ' gdat_data.gdat_params.equil ' ;zshift=' num2str(zshift)];
    gdat_data.units = 'T m^2';
    gdat_data.dimunits = {'m','m','s'};
    gdat_data.request_description = ['data=psi, x=(R,Z), eqdsk contains eqdsk structure with which ' ...
                    'plot_eqdsk, write_eqdsk, read_eqdsk can be used'];
    
   case {'equil'}
    exp_name_eff = gdat_data.gdat_params.exp_name;
    gdat_data.gdat_params.equil = upper(gdat_data.gdat_params.equil);
    DIAG = gdat_data.gdat_params.equil; % 'EQI' by default at this stage, should become EQH?
    M_Rmesh_par = sf2par(DIAG,shot,'M','PARMV');
    M_Rmesh = M_Rmesh_par.value + 1; % nb of points
    N_Zmesh_par = sf2par(DIAG,shot,'N','PARMV');
    N_Zmesh = N_Zmesh_par.value + 1; % nb of points
    NTIME_par = sf2par(DIAG,shot,'NTIME','PARMV');
    NTIME = NTIME_par.value; % nb of points
    Lpf_par = rdaAUG_eff(shot,DIAG,'Lpf',exp_name_eff);
    % since June, nb of time points in EQ results is not consistent with NTIME and time
    % It seems the first NTIME points are correct, so use this explicitely
    NTIME_Lpf = length(Lpf_par.value);
    if (NTIME < NTIME_Lpf)
      if nverbose>=3; disp('WARNING: nb of times points smaller then equil results, use first NTIME points'); end
    elseif (NTIME > NTIME_Lpf)
      if nverbose >= 1
	disp('ERROR: nb of times points LARGER then equil results')
	disp('this is unexpected, so stop there and ask Olivier.Sauter@epfl.ch')
      end
      return
    end
    Lpf_tot = Lpf_par.value(1:NTIME); % nb of points: 100000*nb_SOL points + nb_core
    Lpf_SOL = fix(Lpf_tot/100000);
    Lpf1_t = mod(Lpf_tot,100000)+1; % nb of Lpf points
    % since Lpf depends on time, need to load all first and then loop over time for easier mapping
    [qpsi,e]=rdaAUG_eff(shot,DIAG,'Qpsi',exp_name_eff);
    ndimrho = size(qpsi.data,2);
    if ndimrho==NTIME_Lpf
      % data seems to be transposed
      ndimrho = size(qpsi.data,1);
      itotransposeback = 1; % seems x,time inverted so transpose and exchange .x and .t
    else
      itotransposeback = 0;
    end
    qpsi=adapt_rda(qpsi,NTIME,ndimrho,itotransposeback);
    ijnan=find(isnan(qpsi.value));
    qpsi.value(ijnan)=0;
    [psi_tree,e]=rdaAUG_eff(shot,DIAG,'PFL',exp_name_eff);
    psi_tree=adapt_rda(psi_tree,NTIME,ndimrho,itotransposeback);
    [phi_tree,e]=rdaAUG_eff(shot,DIAG,'TFLx',exp_name_eff);
    phi_tree=adapt_rda(phi_tree,NTIME,ndimrho,itotransposeback);
    [Vol,e]=rdaAUG_eff(shot,DIAG,'Vol',exp_name_eff);
    Vol=adapt_rda(Vol,NTIME,2*ndimrho,itotransposeback);
    [Area,e]=rdaAUG_eff(shot,DIAG,'Area',exp_name_eff);
    Area=adapt_rda(Area,NTIME,2*ndimrho,itotransposeback);
    [Ri,e]=rdaAUG_eff(shot,DIAG,'Ri',exp_name_eff);
    Ri=adapt_rda(Ri,NTIME,M_Rmesh,itotransposeback);
    [Zj,e]=rdaAUG_eff(shot,DIAG,'Zj',exp_name_eff);
    Zj=adapt_rda(Zj,NTIME,N_Zmesh,itotransposeback);
    [PFM_tree,e]=rdaAUG_eff(shot,DIAG,'PFM',exp_name_eff);
    PFM_tree=adaptPFM_rda(PFM_tree,M_Rmesh,N_Zmesh,NTIME);
    [Pres,e]=rdaAUG_eff(shot,DIAG,'Pres',exp_name_eff);
    Pres=adapt_rda(Pres,NTIME,2*ndimrho,itotransposeback);
    [Jpol,e]=rdaAUG_eff(shot,DIAG,'Jpol',exp_name_eff);
    Jpol=adapt_rda(Jpol,NTIME,2*ndimrho,itotransposeback);
    [FFP,e]=rdaAUG_eff(shot,DIAG,'FFP',exp_name_eff);
    if ~isempty(FFP.value)
      FFP=adapt_rda(FFP,NTIME,ndimrho,itotransposeback);
    else
      FFP.value=NaN*ones(NTIME,max(Lpf1_t));
    end
    if strcmp(DIAG,'EQI') || strcmp(DIAG,'EQH')
      [Rinv,e]=rdaAUG_eff(shot,DIAG,'Rinv',exp_name_eff);
      Rinv=adapt_rda(Rinv,NTIME,ndimrho,itotransposeback);
      [R2inv,e]=rdaAUG_eff(shot,DIAG,'R2inv',exp_name_eff);
      R2inv=adapt_rda(R2inv,NTIME,ndimrho,itotransposeback);
      [Bave,e]=rdaAUG_eff(shot,DIAG,'Bave',exp_name_eff);
      Bave=adapt_rda(Bave,NTIME,ndimrho,itotransposeback);
      [B2ave,e]=rdaAUG_eff(shot,DIAG,'B2ave',exp_name_eff);
      B2ave=adapt_rda(B2ave,NTIME,ndimrho,itotransposeback);
      [FTRA,e]=rdaAUG_eff(shot,DIAG,'FTRA',exp_name_eff);
      FTRA=adapt_rda(FTRA,NTIME,ndimrho,itotransposeback);
    else
      Rinv.value=[]; R2inv.value=[]; Bave.value=[]; B2ave.value=[]; FTRA.value=[];
    end
    [LPFx,e]=rdaAUG_eff(shot,DIAG,'LPFx',exp_name_eff);
    LPFx.value=LPFx.value(1:NTIME); LPFx.data=LPFx.value; LPFx.t=LPFx.t(1:NTIME);
    [PFxx,e]=rdaAUG_eff(shot,DIAG,'PFxx',exp_name_eff);
    PFxx=adapt_rda(PFxx,NTIME,max(LPFx.value)+1,itotransposeback);
    [RPFx,e]=rdaAUG_eff(shot,DIAG,'RPFx',exp_name_eff);
    RPFx=adapt_rda(RPFx,NTIME,max(LPFx.value)+1,itotransposeback);
    [zPFx,e]=rdaAUG_eff(shot,DIAG,'zPFx',exp_name_eff);
    zPFx=adapt_rda(zPFx,NTIME,max(LPFx.value)+1,itotransposeback);
    % seems "LCFS" q-value is far too large, limit to some max (when diverted)
    max_qValue = 30.; % Note could just put a NaN on LCFS value since ill-defined when diverted
    for it=1:NTIME
      Lpf1 = Lpf1_t(it);
      % Qpsi and similar data is on (time,radius) with radius being: LCFS..Lpf_points dummy LCFS..SOL part
      % change it to (radial,time) and use only Lpf+1 points up to LCFS
      ijok=find(qpsi.value(:,1)); % note: eqr fills in only odd points radially
      % set NaNs to zeroes
      if qpsi.value(ijok(1),1)<0
	gdat_data.qvalue(:,it) = max(qpsi.value(it,Lpf1:-1:1)',-max_qValue);
      else
	gdat_data.qvalue(:,it) = min(qpsi.value(it,Lpf1:-1:1)',max_qValue);
      end
      % get x values
      gdat_data.psi(:,it)=psi_tree.value(it,Lpf1:-1:1)';
      gdat_data.psi_axis(it)= gdat_data.psi(1,it);
      gdat_data.psi_lcfs(it)= gdat_data.psi(end,it);
      gdat_data.rhopolnorm(:,it) = sqrt(abs((gdat_data.psi(:,it)-gdat_data.psi_axis(it)) ./(gdat_data.psi_lcfs(it)-gdat_data.psi_axis(it))));
      if strcmp(DIAG,'EQR');
	% q value has only a few values and from center to edge, assume they are from central rhopol values on
	% But they are every other point starting from 3rd
	ijk=find(gdat_data.qvalue(:,it)~=0);
	if length(ijk)>2
	  % now shots have non-zero axis values in eqr
	  rhoeff=gdat_data.rhopolnorm(ijk,it);
	  qeff=gdat_data.qvalue(ijk,it); % radial order was already inverted above
	  if ijk(1)>1
	    rhoeff = [0.; rhoeff];
	    qeff = [qeff(1) ;qeff];
	  end
	  ij_nonan=find(~isnan(gdat_data.rhopolnorm(:,it)));
	  qfit = zeros(size(gdat_data.rhopolnorm(:,it)));
	  qfit(ij_nonan)=interpos(rhoeff,qeff,gdat_data.rhopolnorm(ij_nonan,it),-0.01,[1 0],[0 0],[300; ones(size(qeff(1:end-1)))]);
	else
	  qfit = zeros(size(gdat_data.rhopolnorm(:,it)));
	end
	gdat_data.qvalue(:,it) = qfit;
      end    
      % get rhotor values
      gdat_data.phi(:,it) = phi_tree.value(it,Lpf1:-1:1)';
      gdat_data.rhotornorm(:,it) = sqrt(abs(gdat_data.phi(:,it) ./ gdat_data.phi(end,it)));
      % get rhovol values
      gdat_data.vol(:,it)=Vol.value(it,2*Lpf1-1:-2:1)'; 
      gdat_data.dvoldpsi(:,it)=Vol.value(it,2*Lpf1:-2:2)'; % 2nd index are dV/dpsi
      gdat_data.rhovolnorm(:,it) = sqrt(abs(gdat_data.vol(:,it) ./ gdat_data.vol(end,it)));
      gdat_data.area(:,it)=Area.value(it,2*Lpf1-1:-2:1)'; 
      gdat_data.dareadpsi(:,it)=Area.value(it,2*Lpf1:-2:2)'; % 2nd index are dV/dpsi
      gdat_data.Rmesh(:,it) = Ri.value(it,1:M_Rmesh);
      gdat_data.Zmesh(:,it) = Zj.value(it,1:N_Zmesh);
      gdat_data.psi2D(1:M_Rmesh,1:N_Zmesh,it) = PFM_tree.value(1:M_Rmesh,1:N_Zmesh,it);
      gdat_data.pressure(:,it)=Pres.value(it,2*Lpf1-1:-2:1)'; 
      gdat_data.dpressuredpsi(:,it)=Pres.value(it,2*Lpf1:-2:2)'; % 2nd index are dV/dpsi
      if ~isempty(Jpol.value)
	gdat_data.jpol(:,it)=Jpol.value(it,2*Lpf1-1:-2:1)'; 
	gdat_data.djpolpsi(:,it)=Jpol.value(it,2*Lpf1:-2:2)'; % 2nd index are dV/dpsi
      else
	gdat_data.jpol = [];
	gdat_data.djpolpsi = [];
      end
      gdat_data.ffprime(:,it) = FFP.value(it,Lpf1:-1:1)';
      gdat_data.Xpoints.psi(1:LPFx.value(it)+1,it) = PFxx.value(it,1:LPFx.value(it)+1);
      gdat_data.Xpoints.Rvalue(1:LPFx.value(it)+1,it) = RPFx.value(it,1:LPFx.value(it)+1);
      gdat_data.Xpoints.Zvalue(1:LPFx.value(it)+1,it) = zPFx.value(it,1:LPFx.value(it)+1);
      if ~isempty(Rinv.value)
	gdat_data.rinv(:,it) = Rinv.value(it,Lpf1:-1:1)';
      else
	gdat_data.rinv = [];
      end
      if ~isempty(R2inv.value)
	gdat_data.r2inv(:,it) = R2inv.value(it,Lpf1:-1:1)';
      else
	gdat_data.r2inv = [];
      end
      if ~isempty(Bave.value)
	gdat_data.bave(:,it) = Bave.value(it,Lpf1:-1:1)';
      else
	gdat_data.bave = [];
      end
      if ~isempty(B2ave.value)
	gdat_data.b2ave(:,it) = B2ave.value(it,Lpf1:-1:1)';
      else
	gdat_data.b2ave = [];
      end
      if ~isempty(FTRA.value)
	gdat_data.ftra(:,it) = FTRA.value(it,Lpf1:-1:1)';
      else
	gdat_data.ftra = [];
      end
      %
    end
    gdat_data.x = gdat_data.rhopolnorm;
    %
    [equil_Rcoil,e]=rdaAUG_eff(shot,DIAG,'Rcl',exp_name_eff);
    gdat_data.Rcoils=equil_Rcoil.value;
    [equil_Zcoil,e]=rdaAUG_eff(shot,DIAG,'Zcl',exp_name_eff);
    gdat_data.Zcoils=equil_Zcoil.value;
    % get time values
    [equil_time,e]=rdaAUG_eff(shot,DIAG,'time',exp_name_eff);
    gdat_data.t = equil_time.value(1:NTIME);
    %
    [IpiPSI,e]=rdaAUG_eff(shot,DIAG,'IpiPSI',exp_name_eff);
    gdat_data.Ip = IpiPSI.value(1:NTIME);
    %
    gdat_data.data = gdat_data.qvalue; % put q in data
    gdat_data.units=[]; % not applicable
    gdat_data.data_fullpath = [DIAG ' from expname: ' gdat_data.gdat_params.exp_name '; q(rhopolnorm,t) in .data(.x,.t)'];
    gdat_data.cocos = 17; % should check FPP
    gdat_data.dim{1} = gdat_data.x;
    gdat_data.dim{2} = gdat_data.t;
    gdat_data.dimunits{1} = '';
    gdat_data.dimunits{2} = 's';
    
   case {'ne','te'}
    exp_name_eff = gdat_data.gdat_params.exp_name;
    % ne or Te from Thomson data on raw z mesh vs (z,t)
    nodenameeff = [upper(data_request_eff(1)) 'e_c'];
    node_child_nameeff = [upper(data_request_eff(1)) 'e_core'];
    [a,error_status]=rdaAUG_eff(shot,'VTA',nodenameeff,exp_name_eff);
    if isempty(a.data) || isempty(a.t) || error_status>0
      if nverbose>=3;
	a
	disp(['with data_request= ' data_request_eff])
      end
      return
    end
    gdat_data.(lower(node_child_nameeff)).data = a.data;
    gdat_data.(lower(node_child_nameeff)).t = a.t;
    gdat_data.t = a.t;
    if any(strcmp(fieldnames(a),'units'))
      gdat_data.units=a.units;
    end
    [alow,e]=rdaAUG_eff(shot,'VTA',[upper(data_request_eff(1)) 'elow_c'],exp_name_eff);
    [aup,e]=rdaAUG_eff(shot,'VTA',[upper(data_request_eff(1)) 'eupp_c'],exp_name_eff);
    gdat_data.(lower(node_child_nameeff)).error_bar = max(aup.value-gdat_data.(lower(node_child_nameeff)).data,gdat_data.(lower(node_child_nameeff)).data-alow.value);
    [a,error_status]=rdaAUG_eff(shot,'VTA','R_core',exp_name_eff);
    gdat_data.(lower(node_child_nameeff)).r = repmat(a.data,size(gdat_data.(lower(node_child_nameeff)).data,1),1);
    [a,error_status]=rdaAUG_eff(shot,'VTA','Z_core',exp_name_eff);
    gdat_data.(lower(node_child_nameeff)).z = repmat(a.data',1,size(gdat_data.(lower(node_child_nameeff)).data,2));
    gdat_data.dimunits=[{'Z [m]'} ; {'time [s]'}];
    gdat_data.data_fullpath=[data_request_eff ' from VTA/' upper(data_request_eff(1)) 'e_c and ' upper(data_request_eff(1)) 'e_e'];
    nb_core = size(gdat_data.(lower(node_child_nameeff)).z,1);
    gdat_data.data = [];
    gdat_data.data(1:nb_core,:) = gdat_data.(lower(node_child_nameeff)).data(1:nb_core,:);
    gdat_data.dim=[{gdat_data.(lower(node_child_nameeff)).z};{gdat_data.(lower(node_child_nameeff)).t}];
    gdat_data.error_bar(1:nb_core,:) = gdat_data.(lower(node_child_nameeff)).error_bar(1:nb_core,:);

    % add edge part
    nodenameeff_e = [upper(data_request_eff(1)) 'e_e'];
    node_child_nameeff_e = [upper(data_request_eff(1)) 'e_edge'];
    [a,error_status]=rdaAUG_eff(shot,'VTA',nodenameeff_e,exp_name_eff);
    gdat_data.(lower(node_child_nameeff_e)).data = a.data;
    ij_edge_notok = find(a.data>5e21);
    gdat_data.(lower(node_child_nameeff_e)).data(ij_edge_notok) = NaN;
    gdat_data.(lower(node_child_nameeff_e)).t = a.t;
    if ~isempty(a.data)
      [a,error_status]=rdaAUG_eff(shot,'VTA','R_edge',exp_name_eff);
      gdat_data.(lower(node_child_nameeff_e)).r = repmat(a.data,size(gdat_data.(lower(node_child_nameeff_e)).data,1),1);
      [a,error_status]=rdaAUG_eff(shot,'VTA','Z_edge',exp_name_eff);
      gdat_data.(lower(node_child_nameeff_e)).z = repmat(a.data',1,size(gdat_data.(lower(node_child_nameeff_e)).data,2));
      nb_edge = size(gdat_data.(lower(node_child_nameeff_e)).z,1);
      iaaa=iroundos(gdat_data.(lower(node_child_nameeff_e)).t,gdat_data.(lower(node_child_nameeff)).t);
      gdat_data.data(nb_core+1:nb_core+nb_edge,:) = gdat_data.(lower(node_child_nameeff_e)).data(1:nb_edge,iaaa);
      gdat_data.dim{1}(nb_core+1:nb_core+nb_edge,:)=gdat_data.(lower(node_child_nameeff_e)).z(1:nb_edge,iaaa);
      [alow,e]=rdaAUG_eff(shot,'VTA',[upper(data_request_eff(1)) 'elow_e'],exp_name_eff);
      [aup,e]=rdaAUG_eff(shot,'VTA',[upper(data_request_eff(1)) 'eupp_e'],exp_name_eff);
      gdat_data.(lower(node_child_nameeff_e)).error_bar = max(aup.value-gdat_data.(lower(node_child_nameeff_e)).data, ...
	  gdat_data.(lower(node_child_nameeff_e)).data-alow.value);
      gdat_data.error_bar(nb_core+1:nb_core+nb_edge,:) = gdat_data.(lower(node_child_nameeff_e)).error_bar(1:nb_edge,iaaa);
    else
      gdat_data.(lower(node_child_nameeff_e)).data = [];
      gdat_data.(lower(node_child_nameeff_e)).t = [];
      gdat_data.(lower(node_child_nameeff_e)).error_bar = [];
      gdat_data.(lower(node_child_nameeff_e)).r = [];
      gdat_data.(lower(node_child_nameeff_e)).z = [];
    end
    gdat_data.x=gdat_data.dim{1};
    
   case {'ne_rho', 'te_rho'}
    params_eff = gdat_data.gdat_params;
    params_eff.data_request=data_request_eff(1:2); 
    % get raw data
    [gdat_data,params_kin,error_status]=gdat_aug(shot,params_eff);
    if error_status>0
      if nverbose>=3; disp(['problems with ' params_eff.data_request]); end
      return
    end
    % add rho coordinates
    params_eff.data_request='equil'; 
    [equil,params_equil,error_status]=gdat_aug(shot,params_eff);
    if error_status>0
      if nverbose>=3; disp(['problems with ' params_eff.data_request]); end
      return
    end
    gdat_data.gdat_params.equil = params_equil.equil;

    % core
    node_child_nameeff = [upper(data_request_eff(1)) 'e_core'];
    inb_chord_core=size(gdat_data.(lower(node_child_nameeff)).r,1);
    inb_time_core=size(gdat_data.(lower(node_child_nameeff)).r,2);
    psi_out_core = NaN*ones(inb_chord_core,inb_time_core);
    rhopsinorm_out_core = NaN*ones(inb_chord_core,inb_time_core);
    rhotornorm_out_core = NaN*ones(inb_chord_core,inb_time_core);
    rhovolnorm_out_core = NaN*ones(inb_chord_core,inb_time_core);
    % edge
    node_child_nameeff_e = [upper(data_request_eff(1)) 'e_edge'];
    inb_chord_edge=size(gdat_data.(lower(node_child_nameeff_e)).r,1);
    inb_time_edge=size(gdat_data.(lower(node_child_nameeff_e)).r,2);
    psi_out_edge = NaN*ones(inb_chord_edge,inb_time_edge);
    rhopsinorm_out_edge = NaN*ones(inb_chord_edge,inb_time_edge);
    rhotornorm_out_edge = NaN*ones(inb_chord_edge,inb_time_edge);
    rhovolnorm_out_edge = NaN*ones(inb_chord_edge,inb_time_edge);
    % constructs intervals within which a given equil is used: [time_equil(i),time_equil(i+1)]
    time_equil=[1.5*equil.t(1)-0.5*equil.t(2) 0.5.*(equil.t(1:end-1)+equil.t(2:end)) 1.5*equil.t(end)-0.5*equil.t(end-1)];
    for itequil=1:length(time_equil)-1
      rr=equil.Rmesh(:,itequil);
      zz=equil.Zmesh(:,itequil);
      psirz_in = equil.psi2D(:,:,itequil);
      it_core_inequil = find(gdat_data.(lower(node_child_nameeff)).t>=time_equil(itequil) & gdat_data.(lower(node_child_nameeff)).t<=time_equil(itequil+1));
      if ~isempty(it_core_inequil)
	rout_core=gdat_data.(lower(node_child_nameeff)).r(:,it_core_inequil);
	zout_core=gdat_data.(lower(node_child_nameeff)).z(:,it_core_inequil);
	psi_at_routzout = interpos2Dcartesian(rr,zz,psirz_in,rout_core,zout_core);
	psi_out_core(:,it_core_inequil) = reshape(psi_at_routzout,inb_chord_core,length(it_core_inequil));
	rhopsinorm_out_core(:,it_core_inequil) = sqrt((psi_out_core(:,it_core_inequil)-equil.psi_axis(itequil))./(equil.psi_lcfs(itequil)-equil.psi_axis(itequil)));
	for it_cx=1:length(it_core_inequil)
	  rhotornorm_out_core(:,it_core_inequil(it_cx)) = ...
	      interpos(equil.rhopolnorm(:,itequil),equil.rhotornorm(:,itequil),rhopsinorm_out_core(:,it_core_inequil(it_cx)),-3,[2 2],[0 1]);
	  rhovolnorm_out_core(:,it_core_inequil(it_cx)) = ...
	      interpos(equil.rhopolnorm(:,itequil),equil.rhovolnorm(:,itequil),rhopsinorm_out_core(:,it_core_inequil(it_cx)),-3,[2 2],[0 1]);
	end
      end
      % edge
      it_edge_inequil = find(gdat_data.(lower(node_child_nameeff_e)).t>=time_equil(itequil) & gdat_data.(lower(node_child_nameeff_e)).t<=time_equil(itequil+1));
      if ~isempty(it_edge_inequil)
	rout_edge=gdat_data.(lower(node_child_nameeff_e)).r(:,it_edge_inequil);
	zout_edge=gdat_data.(lower(node_child_nameeff_e)).z(:,it_edge_inequil);
	psi_at_routzout = interpos2Dcartesian(rr,zz,psirz_in,rout_edge,zout_edge);
	psi_out_edge(:,it_edge_inequil) = reshape(psi_at_routzout,inb_chord_edge,length(it_edge_inequil));
	rhopsinorm_out_edge(:,it_edge_inequil) = sqrt((psi_out_edge(:,it_edge_inequil)-equil.psi_axis(itequil))./(equil.psi_lcfs(itequil)-equil.psi_axis(itequil)));
	for it_cx=1:length(it_edge_inequil)
	  rhotornorm_out_edge(:,it_edge_inequil(it_cx)) = ...
	      interpos(equil.rhopolnorm(:,itequil),equil.rhotornorm(:,itequil),rhopsinorm_out_edge(:,it_edge_inequil(it_cx)),-3,[2 2],[0 1]);
	  rhovolnorm_out_edge(:,it_edge_inequil(it_cx)) = ...
	      interpos(equil.rhopolnorm(:,itequil),equil.rhovolnorm(:,itequil),rhopsinorm_out_edge(:,it_edge_inequil(it_cx)),-3,[2 2],[0 1]);
	end
      end
    end
    gdat_data.(lower(node_child_nameeff)).psi = psi_out_core;
    gdat_data.(lower(node_child_nameeff)).rhopsinorm = rhopsinorm_out_core;
    gdat_data.(lower(node_child_nameeff)).rhotornorm = rhotornorm_out_core;
    gdat_data.(lower(node_child_nameeff)).rhovolnorm = rhovolnorm_out_core;
    gdat_data.(lower(node_child_nameeff_e)).psi = psi_out_edge;
    gdat_data.(lower(node_child_nameeff_e)).rhopsinorm = rhopsinorm_out_edge;
    gdat_data.(lower(node_child_nameeff_e)).rhotornorm = rhotornorm_out_edge;
    gdat_data.(lower(node_child_nameeff_e)).rhovolnorm = rhovolnorm_out_edge;
    % put values of rhopsinorm for dim{1} by default
    gdat_data.x = gdat_data.(lower(node_child_nameeff)).rhopsinorm;
    iaaa=iroundos(gdat_data.(lower(node_child_nameeff_e)).t,gdat_data.(lower(node_child_nameeff)).t);
    gdat_data.x(inb_chord_core+1:inb_chord_core+inb_chord_edge,:) = gdat_data.(lower(node_child_nameeff_e)).rhopsinorm(:,iaaa);
    gdat_data.dim{1} = gdat_data.x;
    gdat_data.dimunits{1} = 'rhopsinorm';
    gdat_data.data_fullpath = [gdat_data.data_fullpath ' projected on equilibrium ' gdat_data.gdat_params.equil];
    
   otherwise
    warning(['switchcase= ' data_request_eff ' not known in gdat_aug'])
    error_status=901;
    return
  end
  
else
  warning(['AUG method=' mapping_for_aug.method ' not known yet, contact Olivier.Sauter@epfl.ch'])
  error_status=602;
  return
end

gdat_data.mapping_for_aug = mapping_for_aug;
error_status=0;

return

